searchData={"items":[{"type":"module","title":"CliOptions","doc":"An opinionated command line arguments parser.","ref":"CliOptions.html"},{"type":"module","title":"Features - CliOptions","doc":"The main features of `CliOptions` are:\n\n* Schema based validation of command line arguments.\n* Support for common options types and custom validations.\n* Strict validation by default, an error is returned if an invalid.\nargument is passed.\n* Supports required options, default values, aliases, mutually\nexclusive options and much more (check `CliOptions.Schema` for more\ndetails).\n* Auto-generated docs for the supported arguments.","ref":"CliOptions.html#module-features"},{"type":"module","title":"Usage - CliOptions","doc":"The main function of this module is `parse/2` which parses a list\nof command line options and arguments into a tuple of parsed options,\npositional arguments and extra options (anything passed after the `--`\nseparator).\n\n```cli\nschema = [\n  name: [\n    type: :string,\n    required: true \n  ],\n  num: [\n    type: :integer,\n    default: 4,\n    short: \"n\"\n  ] \n]\n\nCliOptions.parse([\"--name\", \"foo\"], schema)\n>>>\n\n# with positional arguments\nCliOptions.parse([\"--name\", \"foo\", \"-n\", \"2\", \"foo.ex\"], schema)\n>>>\n\n# with positional arguments and extra options\nCliOptions.parse([\"--name\", \"foo\", \"-n\", \"2\", \"file.txt\", \"--\", \"-n\", \"1\"], schema)\n>>>\n\n# with invalid options\nCliOptions.parse([\"--user-name\", \"foo\"], schema)\n```\n\n> #### Strict validation {: .warning}\n>\n> Notice that `CliOptions` adheres to a strict options validation approach. This\n> means that an error will be returned in any of the following cases:\n>\n> - An invalid option is provided\n> - An option is provided more times than expected\n> - The option's type is not valid\n> - A required option is not provided\n>\n> ```cli\n> schema = [\n>   number: [type: :integer],\n>   name: [type: :string, required: true]\n> ]\n>\n> # with invalid option\n> CliOptions.parse([\"--name\", \"foo\", \"--invalid\"], schema)\n> >>>\n>\n> # with missing required option\n> CliOptions.parse([], schema)\n> >>>\n>\n> # with invalid type\n> CliOptions.parse([\"--name\", \"foo\", \"--number\", \"asd\"], schema)\n> >>>\n>\n> # with option re-definition\n> CliOptions.parse([\"--name\", \"foo\", \"--name\", \"bar\"], schema)\n> ```","ref":"CliOptions.html#module-usage"},{"type":"function","title":"CliOptions.docs/2","doc":"Returns documentation for the given schema.\n\nYou can use this to inject documentation in your docstrings. For example,\nsay you have your schema in a mix task:\n\n    @options_schema [...]\n\nWith this, you can use `docs/2` to inject documentation:\n\n```markdown","ref":"CliOptions.html#docs/2"},{"type":"function","title":"Supported Options - CliOptions.docs/2","doc":"#{CliOptions.docs(@options_schema)}\"\n```","ref":"CliOptions.html#docs/2-supported-options"},{"type":"function","title":"Options - CliOptions.docs/2","doc":"* `:sort` - if set to `true` the options will be sorted alphabetically.","ref":"CliOptions.html#docs/2-options"},{"type":"function","title":"CliOptions.parse/2","doc":"Parses `argv` according to the provided `schema`.\n\n`schema` can be either a `CliOptions.Schema` struct, or a keyword list. In\nthe latter case it will be first initialized to a `CliOptions.Schema`.\n\nIf the provided CLI arguments are valid, then an `{:ok, parsed}` tuple is\nreturned, where `parsed` is a three element tuple of the form `{opts, args, extra}`,\nwhere:\n\n  * `opts` - the extracted command line options\n  * `args` - a list of the remaining arguments in `argv` as strings\n  * `extra` - a list of unparsed arguments, if applicable.\n\nIf validation fails, an `{:error, message}` tuple will be returned.\n\n> #### Creating the schema at compile time {: .tip}\n>\n> To avoid the extra cost of initializing the schema, it is possible to create\n> the schema once, and then use that valid schema directly. This is done by\n> using the `CliOptions.Schema.new!/1` function first, and then passing the\n> returned schema to `parse/2`.\n>\n> Usually you will define the schema as a module attribute and then use it in\n> your mix task.\n>\n> ```elixir\n> defmodule Mix.Tasks.MyTask do\n>   use Mix.Task\n>\n>   schema = [\n>     user_name: [type: :string],\n>     verbose: [type: :boolean]\n>   ]\n>\n>   @schema CliOptions.Schema.new!(schema)\n>\n>   @impl Mix.Task\n>   def run(argv) do\n>     {opts, args, extra} = CliOptions.parse!(args, @schema)\n>    \n>     # your task code here\n>   end\n> end\n> ```","ref":"CliOptions.html#parse/2"},{"type":"function","title":"Options names and aliases - CliOptions.parse/2","doc":"By default `CliOptions` will use the hyphen-ized version of the option name\nas the long name of the option.\n\n```cli\nCliOptions.parse([\"--user-name\", \"John\"], [user_name: [type: :string]])\n```\n\nAdditionally you are allowed to set a short version (one-letter string) for the\noption.\n\n```cli\nCliOptions.parse([\"-U\", \"John\"], [user_name: [type: :string, short: \"U\"]])\n```\n\nYou are also allowed to specifically set a long name for the option. In this case the\nauto-generated long name from the option name will not valid.\n\n\n```cli\nCliOptions.parse([\"-U\", \"John\"], [user_name: [type: :string, long: \"user\"]])\n>>>\n\nCliOptions.parse(\n  [\"--user\", \"John\"],\n  [user_name: [type: :string, long: \"user\"]]\n)\n```\n\nAdditionally you can provide an arbitrary number of long and short aliases.\n\n```cli\nschema = [\n  user_name: [\n    type: :string,\n    short: \"u\",\n    aliases: [\"user\", \"user_name\"],\n    short_aliases: [\"U\"]\n  ]\n]\n\n# all following argv are equivalent\ninputs = [\n  [\"--user-name\", \"John\"],\n  [\"--user\", \"John\"],\n  [\"--user_name\", \"John\"],\n  [\"-u\", \"John\"],\n  [\"-U\", \"John\"]\n]\n\nfor argv <- inputs, {opts, [], []} = CliOptions.parse!(argv, schema) do\n  opts == [user_name: \"John\"]\nend\n|> Enum.all?()\n```\n\n> #### Strict parsing by default {: .warning}\n>\n> Notice that `CliOptions` will return an error if a not expected option is\n> encountered. Only options defined in the provided schema are allowed. If you\n> need to support arbitrary options you will have to add them after the return\n> separator and handle them in your application code.\n>\n> ```cli\n> schema = [\n>   file: [\n>     type: :string,\n>     required: true \n>   ],\n>   number: [\n>     type: :integer,\n>     short: \"n\"\n>   ] \n> ]\n>\n> # parses valid arguments\n> CliOptions.parse([\"--file\", \"foo.ex\", \"-n\", \"2\"], schema)\n> >>>\n>\n> # error if invalid argument is encountered\n> CliOptions.parse([\"--file\", \"foo.ex\", \"-b\", \"2\"], schema)\n> >>>\n>\n> # you can add extra arguments after the return separator\n> CliOptions.parse([\"--file\", \"foo.ex\", \"--\", \"-b\", \"2\"], schema)\n> ```","ref":"CliOptions.html#parse/2-options-names-and-aliases"},{"type":"function","title":"Option types - CliOptions.parse/2","doc":"By default all options are assumed to be strings. If you set a different `:type` then\nthe option will be casted to that type. The supported types are:\n\n* `:string` - the default, parses the argument as a string\n* `:integer` - parses the argument as an integer\n* `:float` - parses the argument as a float\n* `:atom` - converts the arguments to atoms\n* `:boolean` - parses the argument as a flag, e.g. no option is expected.\n* `:counter` - treats the argument as a flag that increases an associated counter\n\n```cli\nschema = [\n  user: [\n    type: :string,\n  ],\n  age: [\n    type: :integer,\n  ],\n  height: [\n    type: :float,\n  ]\n]\n\n{:ok, options} =\n  CliOptions.parse(\n    [\"--user\", \"John\", \"--age\", \"34\", \"--height\", \"1.75\"],\n    schema\n  )\noptions\n```","ref":"CliOptions.html#parse/2-option-types"},{"type":"function","title":"Counters - CliOptions.parse/2","doc":"In some command line applications you may want to count how many times an option is given. In\nthis case no option argument is expected. Instead every time the option is encountered a counter\nis incremented. You can define such an option by setting the type to `:counter`.\n\n```cli\nschema = [\n  verbosity: [\n    type: :counter,\n    short: \"v\"\n  ]\n]\n\n# counts the number -v flag is given\nCliOptions.parse([\"-v\", \"-v\", \"-v\"], schema)\n>>>\n\n# if not set it is set to zero\nCliOptions.parse([], schema)\n```","ref":"CliOptions.html#parse/2-counters"},{"type":"function","title":"Default values and required options - CliOptions.parse/2","doc":"Options can have default values. If no command line argument is provided then the parsed\noptions will return the default value instead. For example:\n\n```cli\nschema = [\n  verbose: [\n    type: :boolean,\n    default: true\n  ],\n  retries: [\n    type: :integer,\n    default: 1\n  ] \n]\n\nCliOptions.parse([], schema)\n```\n\n> #### Booleans and counters {: .info}\n>\n> Notice that for options of type `:boolean` or `:counter` a default value is always\n> implicitly set.\n>\n> ```cli\n> schema = [verbose: [type: :boolean], level: [type: :counter]]\n>\n> CliOptions.parse([], schema)\n> ```\n\nAdditionally you can mark an option as required. In this case an error will be returned if\nthe option is not present in the command line arguments.\n\n```cli\nschema = [file: [type: :string, required: true]]\n\nCliOptions.parse([], schema)\n>>>\n\nCliOptions.parse([\"--file\", \"foo.ex\"], schema)\n>>>\n```","ref":"CliOptions.html#parse/2-default-values-and-required-options"},{"type":"function","title":"Options with multiple arguments - CliOptions.parse/2","doc":"If you want to pass a cli option multiple times you can set the `:multiple` option set to\n`true`. In this case every time the option is encountered the value will be appended to\nthe previously encountered values.\n\n```cli\nschema = [\n  file: [\n    type: :string,\n    multiple: true,\n    short: \"f\"\n  ],\n  number: [\n    type: :integer,\n    multiple: true,\n    short: \"n\"\n  ]\n]\n\n# all file values are appended to the file option\nCliOptions.parse([\"-f\", \"foo.ex\", \"--file\", \"bar.ex\", \"-n\", \"1\", \"-n\", \"2\"], schema)\n>>>\n\n# notice that if an argument is passed once, the value will still be a list\nCliOptions.parse([\"-f\", \"foo.ex\"], schema)\n>>>\n\n# all passed items are validated based on the expected type \nCliOptions.parse([\"-n\", \"2\", \"-n\", \"xyz\"], schema)\n>>>\n\n# if multiple is not set then an error is returned if an option is passed twice\nCliOptions.parse([\"--file\", \"foo.ex\", \"--file\", \"xyz.ex\"], [file: [type: :string]])\n```","ref":"CliOptions.html#parse/2-options-with-multiple-arguments"},{"type":"function","title":"Return separator - CliOptions.parse/2","doc":"The separator `--` implies options should no longer be processed. Every argument\nafter the return separator will be added in the `extra` field of the response.\n\n```cli\nCliOptions.parse!([\"--\", \"lib\", \"-n\"], [])\n```\n\nNotice that if the remaining arguments contain another return separator this will\nincluded in the extra:\n\n```cli\nCliOptions.parse!([\"--\", \"lib\", \"-n\", \"--\", \"bar\"], [])\n```","ref":"CliOptions.html#parse/2-return-separator"},{"type":"function","title":"CliOptions.parse!/2","doc":"Similar as `parse/2` but raises an `CliOptions.ParseError` exception if invalid\nCLI arguments are given.\n\nIf there are no errors an `{opts, args, extra}` tuple is returned.","ref":"CliOptions.html#parse!/2"},{"type":"function","title":"Examples - CliOptions.parse!/2","doc":"iex> CliOptions.parse!([\"--file\", \"foo.ex\"], [file: [type: :string]])\n    {[file: \"foo.ex\"], [], []}\n\n    iex> CliOptions.parse!([], [file: [type: :string, required: true]])\n    ** (CliOptions.ParseError) option :file is required","ref":"CliOptions.html#parse!/2-examples"},{"type":"type","title":"CliOptions.argv/0","doc":"","ref":"CliOptions.html#t:argv/0"},{"type":"type","title":"CliOptions.extra/0","doc":"","ref":"CliOptions.html#t:extra/0"},{"type":"type","title":"CliOptions.options/0","doc":"","ref":"CliOptions.html#t:options/0"},{"type":"type","title":"CliOptions.parsed_options/0","doc":"","ref":"CliOptions.html#t:parsed_options/0"},{"type":"module","title":"CliOptions.Schema","doc":"The schema for command line options.","ref":"CliOptions.Schema.html"},{"type":"module","title":"Schema Options - CliOptions.Schema","doc":"The following are the options supported in a schema. They are used for validating\npassed command line arguments:\n\n* `:type` - The type of the argument. Can be one of `:string`, `:boolean`, `:integer`, `:float`, `:counter`, `:atom`. If not set defaults\n  to `:string`.\n\n  ```cli\n  schema = [\n    name: [type: :string],\n    retries: [type: :integer],\n    interval: [type: :float],\n    debug: [type: :boolean],\n    verbose: [type: :counter],\n    mode: [type: :atom]\n  ]\n\n  CliOptions.parse!(\n    [\"--name\", \"foo\", \"--retries\", \"3\", \"--interval\", \"2.5\", \"--debug\"],\n    schema\n  )\n  >>>\n\n  CliOptions.parse!(\n    [\"--verbose\", \"--verbose\", \"--mode\", \"parallel\"],\n    schema\n  )\n  ```\n\n  The default value is `:string`.\n\n* `:default` (`t:term/0`) - The default value for the CLI option if that option is not provided in the CLI\n  arguments. This is validated according to the given `:type`.\n\n  ```cli\n  schema = [user_name: [default: \"John\"]]\n\n  # with no option provided\n  CliOptions.parse!([], schema)\n  >>>\n\n  # provided CLI options override the default value\n  CliOptions.parse!([\"--user-name\", \"Jack\"], schema)\n  >>>\n  ```\n\n* `:long` (`t:String.t/0`) - The long name for the option, it is expected to be provided as `--{long_name}`. If\n  not set defaults to the option name itself, casted to string with underscores\n  replaced by dashes. \n\n  ```cli\n  schema = [\n    # long not set, it is set to --user-name\n    user_name: [type: :string],\n    # you can explicitly set a long name with underscores if needed\n    another_user_name: [type: :string, long: \"another_user_name\"]\n  ]\n\n  CliOptions.parse!([\"--user-name\", \"John\", \"--another_user_name\", \"Jack\"], schema)\n  ```\n\n* `:short` (`t:String.t/0`) - An optional short name for the option. It is expected to be a single letter string.\n\n  ```cli\n  schema = [user_name: [short: \"U\"]]\n\n  CliOptions.parse!([\"-U\", \"John\"], schema)\n  ```\n\n* `:aliases` (list of `t:String.t/0`) - Long aliases for the option. It is expected to be a list of strings. \n\n  ```cli\n  schema = [user_name: [aliases: [\"user_name\"]]]\n\n  # with the default long name\n  CliOptions.parse!([\"--user-name\", \"John\"], schema)\n  >>>\n\n  # with an alias\n  CliOptions.parse!([\"--user_name\", \"John\"], schema)\n  ```\n\n  The default value is `[]`.\n\n* `:short_aliases` (list of `t:String.t/0`) - Similar to `:aliases`, but for short names. The default value is `[]`.\n\n* `:doc` (`t:String.t/0` or `false`) - The documentation for the CLI option. Can be any markdown string. This will be\n  used in the automatically generated options documentation. If set to `false`\n  then the option will not be included in the generated docs.\n\n* `:required` (`t:boolean/0`) - Defines if the option is required or not. An exception will be raised if a required\n  option is not provided in the CLI arguments. The default value is `false`.\n\n* `:multiple` (`t:boolean/0`) - If set to `true` an option can be provided multiple times.\n\n  ```cli\n  schema = [project: [multiple: true]]\n\n  CliOptions.parse!([\"--project\", \"foo\", \"--project\", \"bar\"], schema)\n  ```\n\n  The default value is `false`.\n\n* `:allowed` (list of `t:String.t/0`) - A set of allowed values for the option. If any other value is given an exception\n  will be raised during parsing.\n\n* `:deprecated` (`t:String.t/0`) - Defines a message to indicate that the option is deprecated. The message will\n  be displayed as a warning when passing the item.","ref":"CliOptions.Schema.html#module-schema-options"},{"type":"function","title":"CliOptions.Schema.new!/1","doc":"Validates the schema.","ref":"CliOptions.Schema.html#new!/1"},{"type":"function","title":"Examples - CliOptions.Schema.new!/1","doc":"iex> CliOptions.Schema.new!([name: [type: :string, short: \"U\"], verbose: [type: :boolean]])\n    %CliOptions.Schema{\n      schema: [\n        name: [long: \"name\", multiple: false, required: false, short_aliases: [], aliases: [], type: :string, short: \"U\"],\n        verbose: [long: \"verbose\", default: false, multiple: false, required: false, short_aliases: [], aliases: [], type: :boolean]\n      ],\n      long_mappings: %{\"name\" => :name, \"verbose\" => :verbose},\n      short_mappings: %{\"U\" => :name}\n    }\n\n    iex> CliOptions.Schema.new!([name: [type: :invalid]])\n    ** (ArgumentError) invalid schema for :name, invalid value for :type option: expected one of [:string, :boolean, :integer, :float, :counter, :atom], got: :invalid","ref":"CliOptions.Schema.html#new!/1-examples"},{"type":"type","title":"CliOptions.Schema.t/0","doc":"A `CliOptions.Schema` struct.\n\nIncludes the validated schema and the mapping between option names and options.","ref":"CliOptions.Schema.html#t:t/0"},{"type":"exception","title":"CliOptions.ParseError","doc":"An exception raised when parsing option fails.\n\nFor example, see `CliOptions.parse!/2`.","ref":"CliOptions.ParseError.html"},{"type":"extras","title":"Overview","doc":"# CliOptions\n\n[![Hex.pm](https://img.shields.io/hexpm/v/cli_options.svg)](https://hex.pm/packages/cli_options)\n[![hex.pm](https://img.shields.io/hexpm/l/cli_options.svg)](https://hex.pm/packages/cli_options)\n[![Documentation](https://img.shields.io/badge/-Documentation-blueviolet)](https://hexdocs.pm/cli_options/index.html)\n\nAn opinionated command line arguments parser.\n\n`CliOptions` is an alternative to `OptionParser` that allows you to validate CLI\narguments based on a definition. Out of the box you get a polished CLI experience\nincluding:\n\n  * Support for common options types and custom validations.\n  * Strict validation by design, an error is returned if an invalid.\n  argument is passed.\n  * Supports required options, default values, aliases, mutually\n  exclusive options and much more.\n  * Auto-generated docs for the supported arguments.","ref":"readme.html"},{"type":"extras","title":"Usage - Overview","doc":"A definition is a keyword list specifying among other the expected command line arguments,\ntheir types and aliases. For example:\n\n```elixir\nschema = [\n  project: [\n    type: :string,\n    short: \"p\",\n    doc: \"The project name\",\n    required: true\n  ],\n  tags: [\n    type: :string,\n    multiple: true,\n    long: \"tag\",\n    doc: \"Tags to consider\"\n  ],\n  verbose: [\n    type: :boolean,\n    doc: \"Enable verbose logging\",\n  ],\n  timeout: [\n    type: :integer,\n    doc: \"The timeout in seconds\",\n    default: 10\n  ]\n]\n```\n\nNow you can validate some `argv` using `CliOptions.parse/2`.\n\n```elixir\niex> CliOptions.parse([\"-p\", \"foo\", \"--timeout\", \"30\"], schema)\n{:ok, {[project: \"foo\", verbose: false, timeout: 30], [], []}}\n```\n\nIf the input arguments are valid a tuple of the form `{options, args, extra}` is\nreturned. Check the `CliOptions` docs for more details.\n\nIf the input is invalid an error will be returned:\n\n```elixir\n# invalid type\niex> CliOptions.parse([\"-p\", \"foo\", \"--timeout\", \"a30\"], schema)\n{:error, \":timeout expected an integer argument, got: a\"}\n\n# missing required argument\niex> CliOptions.parse([], schema)\n{:error, \"option :project is required\"}\n\n# with undefined argument\niex> CliOptions.parse([\"-p\", \"foo\", \"--other\", \"x\"], schema)\n{:error, \"invalid option other\"}\n```","ref":"readme.html#usage"},{"type":"extras","title":"Auto-generated docs - Overview","doc":"`Cli.Options.docs/2` can be used to automatically generate documentation from a\nvalid schema. You can use it directly in the `@moduledoc` of your mix tasks as\nfollowing:\n\n```elixir\ndefmodule Mix.Tasks.SomeTask do\n  schema = [...]\n\n  @moduledoc \"\"\"\n  Some task.","ref":"readme.html#auto-generated-docs"},{"type":"extras","title":"CLI Options - Overview","doc":"#{CliOptions.docs(schema)}\n  \"\"\"\nend\n```","ref":"readme.html#cli-options"},{"type":"extras","title":"Installation - Overview","doc":"The package can be installed by adding `cli_options` to your list of dependencies\nin `mix.exs`:\n\n```elixir\ndef deps do\n  [\n    {:cli_options, \"~> 0.1.0\"}\n  ]\nend\n```","ref":"readme.html#installation"},{"type":"extras","title":"Acknowledgements - Overview","doc":"* The awesome [`clap`](https://docs.rs/clap/latest/clap/) Rust CLI parser.\n  * [`nimble_options`](https://github.com/dashbitco/nimble_options).","ref":"readme.html#acknowledgements"},{"type":"extras","title":"License - Overview","doc":"Copyright (c) 2023 Panagiotis Nezis, Sportradar\n\n`CliOptions` is released under the MIT License. See the [LICENSE](LICENSE) file for more\ndetails.","ref":"readme.html#license"},{"type":"extras","title":"Changelog","doc":"# Changelog\n\nAll notable changes to this project will be documented in this file.\n\nThe format is based on [Keep a Changelog](https://keepachangelog.com/en/1.0.0/).\n\n## [v0.1.1](https://github.com/sportradar/elixir-workspace/tree/cli_options/v0.1.1) (2024-07-04)","ref":"changelog.html"},{"type":"extras","title":"Added - Changelog","doc":"* If an option is set with `doc: false` it is not included in docs.\n* Support deprecating options with `:deprecated`.\n\n## [v0.1.0](https://github.com/sportradar/elixir-workspace/tree/cli_options/v0.1.0) (2024-05-13)\n\nInitial release.","ref":"changelog.html#added"},{"type":"extras","title":"License","doc":"# MIT License\n\nCopyright (c) 2023 Panagiotis Nezis, Sportradar\n\nPermission is hereby granted, free of charge, to any person obtaining\na copy of this software and associated documentation files (the\n\"Software\"), to deal in the Software without restriction, including\nwithout limitation the rights to use, copy, modify, merge, publish,\ndistribute, sublicense, and/or sell copies of the Software, and to\npermit persons to whom the Software is furnished to do so, subject to\nthe following conditions:\n\nThe above copyright notice and this permission notice shall be\nincluded in all copies or substantial portions of the Software.\n\nTHE SOFTWARE IS PROVIDED \"AS IS\", WITHOUT WARRANTY OF ANY KIND,\nEXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF\nMERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND\nNONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE\nLIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION\nOF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION\nWITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.","ref":"license.html"}],"content_type":"text/markdown","producer":{"name":"ex_doc","version":[48,46,51,52,46,49]}}